library data_state;

import 'package:state_notifier/state_notifier.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

part 'data_state.freezed.dart';

@freezed
abstract class DataState<T> with _$DataState<T> {
  factory DataState(
    @nullable T model, {
    @Default(false) bool isLoading,
    Object exception,
    StackTrace stackTrace,
  }) = _DataState<T>;

  @late
  bool get hasException => exception != null;

  @late
  bool get hasModel => model != null;
}

class DataStateNotifier<T> extends StateNotifier<DataState<T>> {
  DataStateNotifier(DataState<T> initialData)
      : super(initialData ?? DataState<T>(null));

  void Function() onDispose;

  DataState<T> get data => super.state;

  set data(DataState<T> data) {
    super.state = data;
  }

  @override
  @mustCallSuper
  void dispose() {
    if (mounted) {
      super.dispose();
    }
    onDispose?.call();
  }
}
